<?php 

/**
 Setup for Raspberry PI Sense Hat Addon
 http://elinux.org/RPi-Cam-Web-Interface
 
 @Copyright 2015 Stefan Rick
 @author Stefan Rick
 Mail: stefan@rick-software.de
 Web: http://www.netzberater.de
 
 */

class Rpisensehat_Setup extends Service {	
	public $viewname = 'RPi Sense Hat';
	public $scriptPath = '';
	public $intervals = array("2 hours", "4 hours", "1 day", "7 days", "14 days", "1 month", "3 months", "6 months", "1 year");
	
	public function __construct(){		
		parent::__construct();		
		$this->registerLocale(dirname(__FILE__).'/../locale', 'rpisensehat');
		$this->pluginname = _('RPi Sense Hat');				
		$this->scriptPath = dirname(__FILE__).'/../scripts/';
		$this->pname= 'rpisensehat';
		
		if($_REQUEST['graph'] == 1){			
			$this->createGraph();
		}
		
		// Only for Raspberry PI!
		if($this->getSystemUser() != 'pi'){
			$this->view->message[] = _("Only for Raspberry PI!");
			return true;
		}
		
		if($this->checkLicense(true) == false)
			return true;
		
		if($_REQUEST['ajax'] == 1 && $_REQUEST['action'] == 'install'){
			//Function to get Progress of Installation
			$this->_install(1);
			ob_end_clean();
			echo implode('<br />', $this->view->message);
			ob_flush();
			die();
		}
			
		if(isset($_REQUEST['action'])){									
			if($_REQUEST['action'] == 'save'){
				$this->_save();			
			}
			if($_REQUEST['action'] == 'install'){							
				$this->_install();
			}
			if($_REQUEST['action'] == 'start'){
				$this->view->message[] = $this->start($this->pname,'', $statusname = 'python3', $rootstart = true);
			}
				
			if($_REQUEST['action'] == 'stop'){
				$this->view->message[] = $this->stop($this->pname, '', $statusname = 'python3', $rootstop = true);
			}
				
			if($_REQUEST['action'] == 'kill'){
				$this->view->message[] = $this->kill('python3');
			}	
		}
		$this->_getSettings();
		$this->view->pid = $this->status('python3');
		$this->showHelpSidebar();			
	}		
	
	private function _save(){
		$this->_getSettings();
		$this->selectAutostart(isset($_REQUEST['autostart_logging']) ? 1 : 0, false);		
		if($this->view->loginterval != $_REQUEST['loginterval']){
			$this->saveConfigFileParameter('/opt/max2play/options.conf', 'RPISENSEHAT_LOGINTERVAL', intval($_REQUEST['loginterval']));
		}	
		return true;
	}
	
	/**
	 * try to get Version / at least show if it is installed
	 * @return boolean
	 */
	private function _getSettings(){						
		if(trim(shell_exec('dpkg -s sense-hat | grep Status | grep installed | wc -l')) > 0){
			$this->view->installed = true;
		}else{
			$this->view->installed = false;
		}
		
		if(isset($_REQUEST['interval'])){
			$this->view->interval = $_REQUEST['interval'];
		}
		if(isset($_REQUEST['timeback'])){
			$this->view->timeback = $_REQUEST['timeback'];
		}
		//Autostart Sensors		
		$this->view->autostart_logging = $this->checkAutostart($this->pname, false);
		$this->view->loginterval = $this->getConfigFileParameter('/opt/max2play/options.conf', 'RPISENSEHAT_LOGINTERVAL');
		// TODO: add Logfile Size + Delete Option
		
		return true;
	}	
	
	private function _install($ajax = 0){
		if($ajax == 0){
			if($this->getProgressWithAjax('/opt/max2play/cache/install_rpi-sense-hat.txt', 1, 0, 30)){				
				$shellanswer = $this->writeDynamicScript(array($this->scriptPath."install-rpi-sense-hat.sh ".$this->scriptPath." >> /opt/max2play/cache/install_rpi-sense-hat.txt 2>&1 &"), false, true);
			}
		}else{
			$status = $this->getProgressWithAjax('/opt/max2play/cache/install_rpi-sense-hat.txt', 0,0, 30);
			$this->view->message[] = nl2br($status);
			if(strpos($status, 'finished') !== FALSE){
				//Finished Progress - did not delete progressfile yet
				$this->saveConfigFileParameter('/opt/max2play/options.conf', 'RPISENSEHAT_LOGINTERVAL', 30);
				$this->view->message[] = _('INSTALL SUCCESSFUL');
				shell_exec('rm /opt/max2play/cache/install_rpi-sense-hat.txt');				
			}
		}			
	 	return true;
	}
	
	public function showHelpSidebar(){
		global $helpSidebar;
		$helpSidebar['title'] = _t('Help - RPi-Sense-Hat');
		$helpSidebar['content'] = _t('<ul><li>Installer for Raspberry PI Sense Hat. See graph data on humidity, temperature and air pressure!</li></ul>');
		return true;
	}
	
	public function createGraph(){
		ob_end_clean();
		if(isset($_REQUEST['interval'])){
			$interval = $_REQUEST['interval'];
		}else{
			$interval = '2 hours';
		}
		if(isset($_REQUEST['timeback'])){
			$timeback = $_REQUEST['timeback'];
		}else{
			$timeback = '2 hours';
		}
		// get within timespan
		if($timeback){
			$output = shell_exec("awk -vDate=`date -d'now-".$timeback."' +%Y-%m-%dT%H:%M:%S` -vDate2=`date -d'now-".$timeback."+".$interval."' +%Y-%m-%dT%H:%M:%S` ' { if ($1 > Date && $1 < Date2) print $0}' /opt/max2play/cache/sensorData.txt");
		}else
			$output = shell_exec("awk -vDate=`date -d'now-".$interval."' +%Y-%m-%dT%H:%M:%S` ' { if ($1 > Date) print $0}' /opt/max2play/cache/sensorData.txt");
		// remove Data points if there are too many:
		// solution, data point interval, timespan
		$output=explode("\n", $output);
		$dataPoints = count($output);
		if($dataPoints > 100){
			$interval=10;
			$timespan=3600*2;
			$skip=intval(($timespan/$interval) / 100);
			for($i=0;$i<$dataPoints;$i++){
				if($i%$skip != 0){
					unset($output[$i]);
				}
			}
		}
		echo 'Time	Humidity	Temperature	Pressure'."\n";
		echo implode("\n",$output);
		ob_flush();
		die();
	}
		
}

$rpid = new Rpisensehat_Setup();

include_once(dirname(__FILE__).'/../view/setup.php');

